#include <Arduino.h>
#include <Adafruit_NeoPixel.h>
#include <BluetoothSerial.h>

#define PIN 2
#define NUM_LEDS 150
#define FILAS 15
#define COLUMNAS 10

#define LED 2 // Se usa el LED integrado

bool recibiendo = false;
int i = 0;

uint32_t array[FILAS][COLUMNAS];

long ledCounter = -1;

Adafruit_NeoPixel tira(NUM_LEDS, PIN, NEO_GRB + NEO_KHZ800);

BluetoothSerial BT;

void setup()
{
    Serial.begin(9600); // Inicializa el puerto serial para debugging

    tira.begin();
    tira.show();

    BT.begin("ESP32_Control");
    Serial.println("El dispositivo está listo para emparejarse");
}


uint32_t Color(uint8_t r, uint8_t g, uint8_t b) {
    return ((uint32_t)r << 16) | ((uint32_t)g << 8) | b;
  }

void dibujar()
{
    bool voltear = false;
    for (int i = 0; i < FILAS; i++)
    {
        for (int j = 0; j < COLUMNAS; j++)
        {
            int indice = j + (i * COLUMNAS);

            if (voltear)
            {
                indice = (COLUMNAS - 1) - j + (i * COLUMNAS);
            }

            tira.setPixelColor(indice, array[i][j]);
        }

        voltear = !voltear;
    }
    tira.show();
}

void loop()
{

    if (BT.available()) // Se comprueba si hay algo pendiente por recibir
    { 
        digitalWrite(LED, HIGH);
        if (!recibiendo)
        {
            Serial.println("Recibiendo información...");
            recibiendo = true;
            i = 0;
        }
        else if (i == (FILAS * COLUMNAS) - 1)
        {
            recibiendo = false;
        }

        int r = BT.read() & 0xff;
        int g = BT.read() & 0xff;
        int b = BT.read() & 0xff;

        uint32_t color = tira.Color(r, g, b);

        int fila = i / COLUMNAS;
        int columna = i % COLUMNAS;

        Serial.print(i);
        Serial.print(".- [");
        Serial.print(r);
        Serial.print(",");
        Serial.print(g);
        Serial.print(",");
        Serial.print(b);
        Serial.print("]");
        if ((columna + 1) % COLUMNAS == 0) {
            Serial.print("\n");
        } else {
            Serial.print(", ");
        }

        array[fila][columna] = color;

        i++;
    }
    else
    {
        ledCounter++;
        if (ledCounter == 0)
        {
            //Serial.println("Esperando...");
            digitalWrite(LED, HIGH);
        }
        else if (ledCounter == 100000)
        {
            digitalWrite(LED, LOW);
        }
        else if (ledCounter >= 120000)
        {
            ledCounter = -1;
        }
    }

    if (i != 0 && recibiendo == false)
    {
        Serial.println("Listo.");
        i = 0;
        digitalWrite(LED, LOW);

        BT.flush();

        dibujar();
    }
}